﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Media.Effects;

namespace Sheva.Windows.Effects
{
    public class BrightnessContrastShaderEffect : SourceShaderEffect
    {
        private const Int32 BrightnessConstantIndex = 0;
        private const Int32 ContrastConstantIndex = 1;

        public BrightnessContrastShaderEffect(String shaderFilePath)
            : base(shaderFilePath)
        {
            UpdateShaderValue(BrightnessProperty);
            UpdateShaderValue(ContrastProperty);
        }

        public BrightnessContrastShaderEffect(Stream shaderStream)
            : base(shaderStream)
        {
            UpdateShaderValue(BrightnessProperty);
            UpdateShaderValue(ContrastProperty);
        }

        public Double Brightness
        {
            get { return (Double)GetValue(BrightnessProperty); }
            set { SetValue(BrightnessProperty, value); }
        }

        public static readonly DependencyProperty BrightnessProperty =
            DependencyProperty.Register("Brightness", typeof(double), typeof(BrightnessContrastShaderEffect),
                    new UIPropertyMetadata(0d, PixelShaderConstantCallback(ContrastConstantIndex)));


        public Double Contrast
        {
            get { return (double)GetValue(ContrastProperty); }
            set { SetValue(ContrastProperty, value); }
        }

        public static readonly DependencyProperty ContrastProperty =
            DependencyProperty.Register("Contrast", typeof(double), typeof(BrightnessContrastShaderEffect),
                    new UIPropertyMetadata(0d, PixelShaderConstantCallback(ContrastConstantIndex)));
    }
}
