﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Media.Effects;

namespace Sheva.Windows.Effects
{
    static class ShaderHelper
    {
        public static Boolean IsCriticalException(Exception exception)
        {
            return (((((exception is NullReferenceException) ||
                       (exception is StackOverflowException)) ||
                       ((exception is OutOfMemoryException) ||
                        (exception is System.Threading.ThreadAbortException))) ||
                        (exception is System.Runtime.InteropServices.SEHException)) ||
                        (exception is ExecutionEngineException) ||
                (exception is System.Security.SecurityException));
        }

        // This is a common code pattern, so that I wrapped it up in a helper method with the help of 
        // lamda expression.
        public static T TryThrowCriticalException<T>(Func<T> tryAction)
        {
            T result = default(T);
            try
            {
                if (null != tryAction)
                {
                    result = tryAction();
                }
            }
            catch (Exception exception)
            {
                if (IsCriticalException(exception))
                {
                    throw exception;
                }
                result = default(T);
            }

            return result;
        }
    }
}
